@@ -1,9 +1,10 @@
 test_that("add wrong arguments", {
   doc <- read_pptx()
-  expect_error(add_slide(doc, "Title and blah", "Office Theme"))
-  expect_error(add_slide(doc, "Title and Content", "Office Tddheme"))
+  expect_error(add_slide(doc, "Title and blah", "Office Theme"), fixed = TRUE)
+  expect_error(add_slide(doc, "Title and Content", "Office Tddheme"), fixed = TRUE)
 })
 
+
 test_that("add simple elements into placeholder", {
   skip_if_not_installed("doconv")
   skip_if_not(doconv::msoffice_available())
@@ -19,6 +20,7 @@ test_that("add simple elements into placeholder", {
   expect_snapshot_doc(x = doc, name = "pptx-add-simple", engine = "testthat")
 })
 
+
 test_that("add ggplot into placeholder", {
   skip_if_not_installed("doconv")
   skip_if_not_installed("ggplot2")
@@ -45,6 +47,8 @@ test_that("add ggplot into placeholder", {
   )
   expect_snapshot_doc(x = doc, name = "pptx-add-ggplot2", engine = "testthat")
 })
+
+
 test_that("add base plot into placeholder", {
   skip_if_not_installed("doconv")
   skip_if_not(doconv::msoffice_available())
@@ -63,6 +67,7 @@ test_that("add base plot into placeholder", {
   expect_snapshot_doc(x = doc, name = "pptx-add-barplot", engine = "testthat")
 })
 
+
 test_that("add unordered_list into placeholder", {
   skip_if_not_installed("doconv")
   skip_if_not(doconv::msoffice_available())
@@ -87,6 +92,7 @@ test_that("add unordered_list into placeholder", {
   expect_snapshot_doc(x = doc, name = "pptx-add-ul", engine = "testthat")
 })
 
+
 test_that("add block_list into placeholder", {
   skip_if_not_installed("doconv")
   skip_if_not(doconv::msoffice_available())
@@ -114,6 +120,7 @@ test_that("add block_list into placeholder", {
   expect_snapshot_doc(x = doc, name = "pptx-add-blocklist", engine = "testthat")
 })
 
+
 test_that("add formatted par into placeholder", {
   bold_face <- shortcuts$fp_bold(font.size = 30)
   bold_redface <- update(bold_face, color = "red")
@@ -152,6 +159,7 @@ test_that("add xml into placeholder", {
   expect_equal(sm[2, ]$text, "Hello world 1")
 })
 
+
 test_that("slidelink shape", {
   doc <- read_pptx()
   doc <- add_slide(doc, layout = "Title and Content", master = "Office Theme")
@@ -271,103 +279,5 @@ test_that("empty_content in pptx", {
   expect_equal(slide_summary(doc)$cx, 2)
 })
 
-test_that("pptx ph locations", {
-  doc <- read_pptx()
-  doc <- add_slide(doc, "Title and Content", "Office Theme")
-
-  doc <- ph_with(
-    x = doc, value = "left",
-    location = ph_location_left()
-  )
-  doc <- ph_with(
-    x = doc, value = "right",
-    location = ph_location_right()
-  )
-  doc <- ph_with(
-    x = doc, value = "title",
-    location = ph_location_type(type = "title")
-  )
-  doc <- ph_with(
-    x = doc, value = "fullsize",
-    location = ph_location_fullsize()
-  )
-  doc <- ph_with(
-    x = doc, value = "from title",
-    location = ph_location_template(
-      left = 1, width = 2, height = 1, top = 4,
-      type = "title", newlabel = "newlabel"
-    )
-  )
-
-  layouts_info <- layout_properties(doc)
-
-
-  title_xfrm <- layouts_info[layouts_info$name %in% "Two Content" &
-    layouts_info$type %in% "title", c("offx", "offy", "cx", "cy")]
-  side_xfrm <- layouts_info[layouts_info$name %in% "Two Content" &
-    layouts_info$type %in% "body", c("offx", "offy", "cx", "cy")]
-  full_xfrm <- as.data.frame(slide_size(doc))
-  names(full_xfrm) <- c("cx", "cy")
-  full_xfrm <- cbind(data.frame(offx = 0L, offy = 0L), full_xfrm)
-  from_title_xfrm <- data.frame(offx = 1, offy = 4, cx = 2, cy = 1)
-  theorical_xfrm <- rbind(
-    side_xfrm,
-    title_xfrm,
-    full_xfrm,
-    from_title_xfrm
-  )
-
-  all_xfrm <- xml_find_all(
-    x = doc$slide$get_slide(1)$get(),
-    xpath = "/p:sld/p:cSld/p:spTree/p:sp/p:spPr/a:xfrm"
-  )
-  offx <- xml_attr(xml_child(all_xfrm, "a:off"), "x")
-  offx <- as.integer(offx) / 914400
-  offy <- xml_attr(xml_child(all_xfrm, "a:off"), "y")
-  offy <- as.integer(offy) / 914400
-  cx <- xml_attr(xml_child(all_xfrm, "a:ext"), "cx")
-  cx <- as.integer(cx) / 914400
-  cy <- xml_attr(xml_child(all_xfrm, "a:ext"), "cy")
-  cy <- as.integer(cy) / 914400
-
-  observed_xfrm <- data.frame(offx = offx, offy = offy, cx = cx, cy = cy)
-  expect_equivalent(observed_xfrm, theorical_xfrm)
-})
-
-test_that("pptx ph labels", {
-  doc <- read_pptx()
-  doc <- add_slide(doc, "Title and Content", "Office Theme")
-
-  doc <- ph_with(
-    x = doc, value = "elephant",
-    location = ph_location_type(newlabel = "label1")
-  )
-  doc <- ph_with(
-    x = doc, value = "elephant",
-    location = ph_location_label(
-      ph_label = "Date Placeholder 3",
-      newlabel = "label2"
-    )
-  )
-  doc <- ph_with(
-    x = doc, value = "elephant",
-    location = ph_location_left(newlabel = "label3")
-  )
-  doc <- ph_with(
-    x = doc, value = "elephant",
-    location = ph_location_right(newlabel = "label4")
-  )
-
-  all_nvpr <- xml_find_all(
-    x = doc$slide$get_slide(1)$get(),
-    xpath = "/p:sld/p:cSld/p:spTree/p:sp/p:nvSpPr/p:cNvPr"
-  )
-  expect_equal(
-    xml_attr(all_nvpr, "name"),
-    paste0("label", 1:4)
-  )
-})
-
-
 
 unlink("*.pptx")
